@@ -281,6 +281,88 @@ static bool parseExtraField(const char *buffer, int size, bool islocal,
   return true;
 }
 
+/**
+ * Checks if a token for a central or local header has been found and resets
+ * the device to the begin of the token. If a token for the data descriptor is
+ * found it is assumed there is a central or local header token starting right
+ * behind the data descriptor, and the device is set accordingly to the begin
+ * of that token.
+ * To be called when a 'P' has been found.
+ * @param buffer start of buffer with the 3 bytes behind 'P'
+ * @param dev device that is read from
+ * @return true if a local or central header begin is or could be reached
+ */
+static bool handlePossibleHeaderBegin(const char* buffer, QIODevice *dev)
+{
+    // we have to detect three magic tokens here:
+    // PK34 for the next local header in case there is no data descriptor
+    // PK12 for the central header in case there is no data descriptor
+    // PK78 for the data descriptor in case it is following the compressed data
+    // TODO: optimize using 32bit const data for comparison instead of byte-wise,
+    // given we run at least on 32bit CPUs
+
+    if (buffer[0] == 'K') {
+        if (buffer[1] == 7 && buffer[2] == 8) {
+            // data descriptor token found
+            dev->seek(dev->pos() + 12); // skip the 'data_descriptor'
+            return true;
+        }
+
+        if ((buffer[1] == 1 && buffer[2] == 2)
+            || (buffer[1] == 3 && buffer[2] == 4)) {
+            // central/local header token found
+            dev->seek(dev->pos() - 4);
+            // go back 4 bytes, so that the magic bytes can be found
+            // in the next cycle...
+            return true;
+        }
+    }
+    return false;
+}
+
+/**
+ * Reads the device forwards from the current pos until a token for a central or
+ * local header has been found or is to be assumed.
+ * @param dev device that is read from
+ * @return true if a local or central header token could be reached, false on error
+ */
+static bool seekToNextHeaderToken(QIODevice *dev)
+{
+    bool headerTokenFound = false;
+    char buffer[3];
+
+    while (!headerTokenFound) {
+        int n = dev->read(buffer, 1);
+        if (n < 1) {
+            //qWarning() << "Invalid ZIP file. Unexpected end of file. (#2)";
+            return false;
+        }
+
+        if (buffer[0] != 'P') {
+            continue;
+        }
+
+        n = dev->read(buffer, 3);
+        if (n < 3) {
+            //qWarning() << "Invalid ZIP file. Unexpected end of file. (#3)";
+            return false;
+        }
+
+        if (handlePossibleHeaderBegin(buffer, dev)) {
+            headerTokenFound = true;
+        } else {
+            for (int i = 0; i < 3; ++i) {
+                if (buffer[i] == 'P') {
+                    // We have another P character so we must go back a little to check if it is a magic
+                    dev->seek(dev->pos() - 3 + i);
+                    break;
+                }
+            }
+        }
+    }
+    return true;
+}
+
 ////////////////////////////////////////////////////////////////////////
 /////////////////////////// KZip ///////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////
@@ -444,56 +526,15 @@ bool KZip::openArchive( QIODevice::OpenMode mode )
             {
 	    	// here we have to read through the compressed data to find
 		// the next PKxx
-	        //kDebug(7040) << "trying to seek for next PK78";
-                bool foundSignature = false;
-
-                while (!foundSignature)
-                {
-                    n = dev->read( buffer, 1 );
-                    if (n < 1)
-                    {
-                        kWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#2)";
-                        return false;
-                    }
-
-                    if ( buffer[0] != 'P' )
-                        continue;
-
-                    n = dev->read( buffer, 3 );
-                    if (n < 3)
-                    {
-                        kWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#3)";
-                        return false;
-                    }
-
-                    // we have to detect three magic tokens here:
-		    // PK34 for the next local header in case there is no data descriptor
-		    // PK12 for the central header in case there is no data descriptor
-		    // PK78 for the data descriptor in case it is following the compressed data
-
-		    if ( buffer[0] == 'K' && buffer[1] == 7 && buffer[2] == 8 )
-                    {
-                        foundSignature = true;
-                        dev->seek( dev->pos() + 12 ); // skip the 'data_descriptor'
-                    }
-		    else if ( ( buffer[0] == 'K' && buffer[1] == 1 && buffer[2] == 2 )
-		         || ( buffer[0] == 'K' && buffer[1] == 3 && buffer[2] == 4 ) )
-                    {
-                        foundSignature = true;
-                        dev->seek( dev->pos() - 4 ); // go back 4 bytes, so that the magic bytes can be found...
-                    }
-                    else if ( buffer[0] == 'P' || buffer[1] == 'P' || buffer[2] == 'P' )
-                    {
-                        // We have another P character so we must go back a little to check if it is a magic
-                        dev->seek( dev->pos() - 3 );
-                    }
-
+                if (!seekToNextHeaderToken(dev)) {
+                    return false;
                 }
             }
             else
             {
 	    	// here we skip the compressed data and jump to the next header
 	        //kDebug(7040) << "general purpose bit flag indicates, that local file header contains valid size";
+                bool foundSignature = false;
 		// check if this could be a symbolic link
 		if (compression_mode == NoCompression
 	    		&& uncomp_size <= max_path_len
@@ -511,47 +552,11 @@ bool KZip::openArchive( QIODevice::OpenMode mode )
 		    {
 		    	// here we cannot trust the compressed size, so scan through the compressed
 			// data to find the next header
-			bool foundSignature = false;
 
-			while (!foundSignature)
-			{
-				n = dev->read( buffer, 1 );
-				if (n < 1)
-				{
-					kWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#2)";
-					return false;
-				}
-
-				if ( buffer[0] != 'P' )
-					continue;
-
-				n = dev->read( buffer, 3 );
-				if (n < 3)
-				{
-					kWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#3)";
-					return false;
-				}
-
-				// we have to detect three magic tokens here:
-				// PK34 for the next local header in case there is no data descriptor
-				// PK12 for the central header in case there is no data descriptor
-				// PK78 for the data descriptor in case it is following the compressed data
-
-				if ( buffer[0] == 'K' && buffer[1] == 7 && buffer[2] == 8 )
-				{
-					foundSignature = true;
-					dev->seek( dev->pos() + 12 ); // skip the 'data_descriptor'
-				}
-
-				if ( ( buffer[0] == 'K' && buffer[1] == 1 && buffer[2] == 2 )
-					|| ( buffer[0] == 'K' && buffer[1] == 3 && buffer[2] == 4 ) )
-				{
-					foundSignature = true;
-					dev->seek( dev->pos() - 4 );
-					// go back 4 bytes, so that the magic bytes can be found
-					// in the next cycle...
-				}
-			}
+                        if (!seekToNextHeaderToken(dev)) {
+                            return false;
+                        }
+                        foundSignature = true;
 		    }
 		    else
 		    {
@@ -567,6 +572,21 @@ bool KZip::openArchive( QIODevice::OpenMode mode )
 		    }
 
 		}
+                // test for optional data descriptor
+                if (!foundSignature) {
+//                     qDebug() << "Testing for optional data descriptor";
+                    // read static data descriptor
+                    n = dev->read(buffer, 4);
+                    if (n < 4) {
+//                         qWarning() << "Invalid ZIP file. Unexpected end of file. (#1)";
+                        return false;
+                    }
+
+                    if (buffer[0] != 'P' || !handlePossibleHeaderBegin(buffer+1, dev)) {
+                        // assume data descriptor without signature
+                        dev->seek(dev->pos() + 8); // skip rest of the 'data_descriptor'
+                    }
+                }
 
 // not needed any more
 /*                // here we calculate the length of the file in the zip
@@ -766,11 +786,14 @@ bool KZip::openArchive( QIODevice::OpenMode mode )
                 {
                     foundSignature = true;
                     dev->seek( dev->pos() - 4 ); // go back 4 bytes, so that the magic bytes can be found...
-                }
-                else if ( buffer[0] == 'P' || buffer[1] == 'P' || buffer[2] == 'P' )
-                {
-                        // We have another P character so we must go back a little to check if it is a magic
-                    dev->seek( dev->pos() - 3 );
+                } else {
+                    for (int i = 0; i < 3; ++i) {
+                        if (buffer[i] == 'P') {
+                            // We have another P character so we must go back a little to check if it is a magic
+                            dev->seek(dev->pos() - 3 + i);
+                            break;
+                        }
+                    }
                 }
             }
         }
@@ -1010,24 +1033,6 @@ bool KZip::doPrepareWriting(const QString &name, const QString &user,
         return false;
     }
 
-    // delete entries in the filelist with the same fileName as the one we want
-    // to save, so that we don't have duplicate file entries when viewing the zip
-    // with konqi...
-    // CAUTION: the old file itself is still in the zip and won't be removed !!!
-    QMutableListIterator<KZipFileEntry*> it( d->m_fileList );
-	//kDebug(7040) << "fileName to write: " << name;
-    while(it.hasNext())
-    {
-	    it.next();
-    	//kDebug(7040) << "prepfileName: " << it.current()->path();
-		if (name == it.value()->path() )
-        {
-	    	//kDebug(7040) << "removing following entry: " << it.current()->path();
-		delete it.value();
-	        it.remove();
-        }
-
-    }
     // Find or create parent dir
     KArchiveDirectory* parentDir = rootDir();
     QString fileName( name );
@@ -1039,6 +1044,27 @@ bool KZip::doPrepareWriting(const QString &name, const QString &user,
         parentDir = findOrCreate( dir );
     }
 
+    // delete entries in the filelist with the same fileName as the one we want
+    // to save, so that we don't have duplicate file entries when viewing the zip
+    // with konqi...
+    // CAUTION: the old file itself is still in the zip and won't be removed !!!
+    QMutableListIterator<KZipFileEntry*> it( d->m_fileList );
+	//kDebug(7040) << "fileName to write: " << name;
+    while(it.hasNext())
+    {
+        it.next();
+        //kDebug(7040) << "prepfileName: " << it.current()->path();
+        if (name == it.value()->path() )
+        {
+            // also remove from the parentDir
+            parentDir->removeEntry(it.value());
+            //kDebug(7040) << "removing following entry: " << it.current()->path();
+            delete it.value();
+            it.remove();
+        }
+
+    }
+
     // construct a KZipFileEntry and add it to list
     KZipFileEntry * e = new KZipFileEntry( this, fileName, perm, mtime, user, group, QString(),
                                            name, device()->pos() + 30 + name.length(), // start
