@@ -792,7 +792,7 @@ KMimeType::Ptr KFileItem::determineMimeType() const
         d->m_bMimeTypeKnown = true;
     }
 
-    if (isSlow() && d->m_delayedMimeTypes) { // if we delayed getting the iconName up till now, this is the right point in time to do so
+    if (d->m_delayedMimeTypes) { // if we delayed getting the iconName up till now, this is the right point in time to do so
         d->m_delayedMimeTypes = false;
         d->m_useIconNameCache = false;
         (void)iconName();
@@ -817,7 +817,7 @@ bool KFileItem::isFinalIconKnown() const
     if (!d) {
         return false;
     }
-    return d->m_bMimeTypeKnown && (!d->m_delayedMimeTypes || !isSlow());
+    return d->m_bMimeTypeKnown && (!d->m_delayedMimeTypes);
 }
 
 QString KFileItem::mimeComment() const
@@ -902,7 +902,7 @@ QString KFileItem::iconName() const
         mime = mimeTypePtr();
     }
 
-    const bool delaySlowOperations = isSlow() && d->m_delayedMimeTypes;
+    const bool delaySlowOperations = d->m_delayedMimeTypes;
 
     if (isLocalUrl && !delaySlowOperations && mime->is("application/x-desktop")) {
         d->m_iconName = iconFromDesktopFile(url.toLocalFile());
@@ -959,8 +959,7 @@ QStringList KFileItem::overlays() const
         names.append("emblem-symbolic-link");
     }
 
-    if ( !S_ISDIR( d->m_fileMode ) // Locked dirs have a special icon, use the overlay for files only
-         && !isReadable()) {
+    if (!isReadable()) {
         names.append("object-locked");
     }
 
@@ -1000,11 +999,6 @@ QStringList KFileItem::overlays() const
     }
 #endif  // Q_OS_WIN
 
-    if ( d->m_pMimeType && d->m_url.fileName().endsWith( QLatin1String( ".gz" ) ) &&
-         d->m_pMimeType->is("application/x-gzip") ) {
-        names.append("application-zip");
-    }
-
     return names;
 }
 
