@@ -30,13 +30,10 @@
 #include <QtGui/QTextLayout>
 #include <QtGui/QTextOption>
 
-#include <plasma/plasma_export.h>
-#include <plasma/framesvg.h>
-#include <plasma/theme.h>
-#include <plasma/svg.h>
-
 #include "iconwidget.h"
+#include "framesvg.h"
 #include "private/actionwidgetinterface_p.h"
+#include "theme.h"
 
 class QAction;
 class QPainter;
@@ -159,11 +156,9 @@ public:
     QBrush backgroundBrush(const QStyleOptionGraphicsItem *option) const;
 
     QString elidedText(QTextLayout &layout,
-                       const QStyleOptionGraphicsItem *option,
                        const QSizeF &maxSize) const;
 
     QSizeF layoutText(QTextLayout &layout,
-                      const QStyleOptionGraphicsItem *option,
                       const QString &text, const QSizeF &constraints) const;
 
     QSizeF layoutText(QTextLayout &layout, const QString &text,
@@ -216,13 +211,12 @@ public:
     void readColors();
     void colorConfigChanged();
     void iconConfigChanged();
-    QFont widgetFont() const;
     void hoverAnimationFinished();
     void init();
     void layoutIcons(const QStyleOptionGraphicsItem *option);
+    QSizeF iconSizeForWidgetSize(const QStyleOptionGraphicsItem *option, const QSizeF &widgetSize);
     void animateMainIcon(bool, const IconWidgetStates state);
 
-    IconWidget *q;
     QString text;
     QString infoText;
     Svg *iconSvg;
@@ -280,7 +274,7 @@ void IconWidgetPrivate::setLayoutOptions(QTextLayout &layout,
 
     textoption.setWrapMode(QTextOption::WordWrap);  // NOTE: assumption as well
 
-    layout.setFont(widgetFont());
+    layout.setFont(q->font());
     layout.setTextOption(textoption);
 }
 
